/*-------------------------------------+-------------------------------------*/
/*                                  lansan.c                                 */

#include "hardbord.h"

static boolean match (const char * s, const char * l)
{
   while (*s) {
      while (*s != *l)
         if (! *l)
            return FALSE;
         else
            l++;
      s++, l++;
   }
   return TRUE;
}

boolean hardboard::equal (const char * s, const char * l)
{
   while (*s)
      if (*s++ != *l++)
         return FALSE;
   if (! *l || l[0]=='=' && l[1]==lang[QUEEN] || *l=='+' || *l=='#')
      return TRUE;
   else
      return FALSE;
}

unsigned hardboard::lanhit (const char * s, unsigned * movenum)
{
   unsigned result = 0;

   for (unsigned n=0; n<n_moves; n++)
      if (match (s, lanlist[n]))
         *movenum=n, result++;
   return result;
}

unsigned hardboard::sanhit (const char * s, unsigned * movenum)
{
   unsigned result = 0;

   for (unsigned n=0; n<n_moves; n++)
      if (equal (s, sanlist[n]))
         *movenum=n, result++;
   if (! result)
      for (unsigned n=0; n<n_moves; n++)
         if (match (s, sanlist[n]))
            *movenum=n, result++;
   return result;
}

void hardboard::make_lanlist (void)
{
   delete lanlist;
   lanlist = new char[n_moves][11];

   unsigned n = 0;
   for (move * m=movelist; m; m=m->next) {
      unsigned i = 0;
      lanlist[n][i++] = lang[m->p0->type];
      lanlist[n][i++] = (m->p0->loc-board) % 8 + 'a';
      lanlist[n][i++] = (m->p0->loc-board) / 8 + '1';
      lanlist[n][i++] = (m->f1->occ ||
         m->f1==specials.epfield && ! m->p0->type) ? 'x' : '-';
      lanlist[n][i++] = (m->f1->occ) ? lang[m->f1->occ->type] : ' ';
      lanlist[n][i++] = (m->f1-board) % 8 + 'a';
      lanlist[n][i++] = (m->f1-board) / 8 + '1';
      if (m->promotion) {
         lanlist[n][i++] = '=';
         lanlist[n][i++] = lang[m->promotion];
      }
      if (m->check)
         if (m->mate)
            lanlist[n][i++] = '#';
         else
            lanlist[n][i++] = '+';
      lanlist[n][i++] = '\0';
      n++;
   }
   return;
}

void hardboard::make_sanlist (void)
{
   delete sanlist;
   sanlist = new char[n_moves][8];

   for (unsigned n=0; n<n_moves; n++)
      if (match ("Ke1-g1", lanlist[n]) || match ("Ke8-g8", lanlist[n])) {
         sanlist[n][0] = 'O';
         sanlist[n][1] = '-';
         sanlist[n][2] = 'O';
         sanlist[n][3] = lanlist[n][7];
         sanlist[n][4] = '\0';
      } else if (match ("Ke1-c1",lanlist[n]) || match ("Ke8-c8",lanlist[n])) {
         sanlist[n][0] = 'O';
         sanlist[n][1] = '-';
         sanlist[n][2] = 'O';
         sanlist[n][3] = '-';
         sanlist[n][4] = 'O';
         sanlist[n][5] = lanlist[n][7];
         sanlist[n][6] = '\0';
      } else if (lanlist[n][0] == lang[PAWN]) {
         unsigned i = 0;
         if (lanlist[n][3] == 'x') {
            sanlist[n][i++] = lanlist[n][1];
            sanlist[n][i++] = 'x';
         }
         sanlist[n][i++] = lanlist[n][5];
         sanlist[n][i++] = lanlist[n][6];
         sanlist[n][i++] = lanlist[n][7];
         sanlist[n][i++] = lanlist[n][8];
         sanlist[n][i++] = lanlist[n][9];
         sanlist[n][i++] = '\0';
      } else {
         unsigned i = 0;
         sanlist[n][i++] = lanlist[n][0];
         sanlist[n][i++] = lanlist[n][3];
         sanlist[n][i++] = lanlist[n][5];
         sanlist[n][i++] = lanlist[n][6];
         sanlist[n][i++] = lanlist[n][7];
         sanlist[n][i++] = '\0';
         if (lanhit (sanlist[n], &i) > 1) {
            i = 0;
            sanlist[n][i++] = lanlist[n][0];
            sanlist[n][i++] = lanlist[n][1];
            sanlist[n][i++] = lanlist[n][3];
            sanlist[n][i++] = lanlist[n][5];
            sanlist[n][i++] = lanlist[n][6];
            sanlist[n][i++] = lanlist[n][7];
            sanlist[n][i++] = '\0';
            if (lanhit (sanlist[n], &i) > 1) {
               i = 0;
               sanlist[n][i++] = lanlist[n][0];
               sanlist[n][i++] = lanlist[n][2];
               sanlist[n][i++] = lanlist[n][3];
               sanlist[n][i++] = lanlist[n][5];
               sanlist[n][i++] = lanlist[n][6];
               sanlist[n][i++] = lanlist[n][7];
               sanlist[n][i++] = '\0';
               if (lanhit (sanlist[n], &i) > 1) {
                  i = 0;
                  sanlist[n][i++] = lanlist[n][0];
                  sanlist[n][i++] = lanlist[n][1];
                  sanlist[n][i++] = lanlist[n][2];
                  sanlist[n][i++] = lanlist[n][3];
                  sanlist[n][i++] = lanlist[n][5];
                  sanlist[n][i++] = lanlist[n][6];
                  sanlist[n][i++] = lanlist[n][7];
                  sanlist[n][i++] = '\0';
               }
            }
         }
         i = 1;
         while (sanlist[n][i] && sanlist[n][i]!='-')
            i++;
         while (sanlist[n][i++])
            sanlist[n][i-1] = sanlist[n][i];
      }
   return;
}

